angular.module('odoo')

    /**
     * -----------------------------------------------------------------------------
     *  Description:    Customer list controller
     * -----------------------------------------------------------------------------
     */
    .controller('CustomersController', function (
        $scope,
        $ionicModal,
        $ionicActionSheet,
        $ionicPopup,
        customersRemoteFactory,
        customersStorageFactory,
        sqlFactory,
        deviceFactory
    ) {
        $scope.loading = false;
        $scope.customers = [];
        $scope.customer = {};

        $ionicModal.fromTemplateUrl('templates/sales/customer.html', {
            scope: $scope
        }).then(function (modal) {
            $scope.customerModal = modal;
        });

        /**
         *
         */
        $scope.$on('$ionicView.enter', function () {
            $scope.fill(false);
        });

        /**
         *
         */
        $scope.$on('$destroy', function () {
            $scope.customerModal.remove();
        });

        /**
         *
         */
        $scope.$on('modal.hidden', function () {
            $scope.customer = {};
        });

        /**
         *
         */
        $scope.$on('device.shaked', function () {
            if ($scope.customerModal.isShown()) {

                deviceFactory.getCurrentPosition(function (position) {
                    $scope.customer.partner_latitude = position.coords.latitude;
                    $scope.customer.partner_longitude = position.coords.longitude;
                    $scope.customer.date_localization = new Date(position.timestamp).toLocaleString();
                }, function (err) {
                    console.log(err);
                });

            } else {
                $scope.fill(false);
            }
        });

        /**
         *
         */
        $scope.fill = function (refresh) {
            $scope.loading = !refresh;

            customersRemoteFactory.sync(function (customers) {
                console.log('Customers sync correctly and receive ' + customers.length + ' items');

                $scope.getCustomers(function () {
                    $scope.$broadcast('scroll.refreshComplete');
                    $scope.loading = false;
                }, function (getCustomersError) {

                    $scope.$broadcast('scroll.refreshComplete');
                    $scope.loading = false;

                    $ionicPopup.alert({ title: 'No se pudo obtener los clientes' });
                });
            }, function (syncErr) {

                $scope.getCustomers(function () {
                    $scope.$broadcast('scroll.refreshComplete');
                    $scope.loading = false;
                }, function (getCustomersError) {

                    $scope.$broadcast('scroll.refreshComplete');
                    $scope.loading = false;

                    $ionicPopup.alert({ title: 'No se pudo obtener los clientes' });
                });
            });
        }

        /**
         *
         */
        $scope.getCustomers = function (success, error) {
            sqlFactory.selectByConstraint('partner', 'customer = 1 AND modified != 2', function (customers) {
                $scope.customers = [];

                for (var i = 0; i < customers.length; i++) {
                    $scope.customers.push(customers.item(i));
                }

                $scope.$apply();
                success();
            }, function (customerGetByConstraintError) {
                error(customerGetByConstraintError);
            });
        }

        /**
         *
         */
        $scope.openOptions = function (index) {
            if (index == -1) {
                $scope.customer = {};
            } else {
                $scope.customer = $scope.customers[index];
            }

            console.log('Customer selected => ' + angular.toJson($scope.customer, true));

            $ionicActionSheet.show({
                titleText: 'Acciones',
                buttons: [
                    {
                        text: '<i class="icon ion-arrow-expand positive"></i> Abrir'
                    },
                    {
                        text: '<i class="icon ion-archive positive"></i> Agregar a contactos'
                    },
                    {
                        text: '<i class="icon ion-navigate positive"></i> Ir a la ubicación'
                    }
                ],
                destructiveText: '<i class="icon ion-trash-a assertive"></i> Eliminar',
                cancel: function () {
                    $scope.customer = {};
                    console.log('ActionSheet canceled');
                },
                buttonClicked: function (index) {
                    switch (index) {
                        case 0:
                            $scope.show();
                            break;
                        case 1:
                            $scope.addContact();
                            break;
                        case 2:
                            $scope.navigate();
                            break;
                        default:
                            $scope.show();
                    }
                    return true;
                },
                destructiveButtonClicked: function () {
                    $scope.delete();

                    return true;
                }
            });
        }

        /**
         *
         */
        $scope.save = function () {
            customersStorageFactory.save($scope.customer, function (customerId) {
                if (!$scope.customer.id) {
                    $scope.customer.id = customerId;
                    $scope.customers.push($scope.customer);
                }

                $scope.customer = {};
                $scope.customerModal.hide();
                console.log('Customer saved');
            }, function (error) {
                $ionicPopup.alert({ title: 'No se ha podido guardar el cliente', template: JSON.stringify(error) });
                console.log(JSON.stringify(error));
            });
        }

        /**
         *
         */
        $scope.delete = function () {
            $ionicPopup.confirm({
                title: 'Confirmar',
                template: 'Estás seguro que quieres eliminar este cliente?'
            }).then(function (confirmation) {
                if (confirmation) {
                    customersStorageFactory.remove($scope.customer, function (affected) {
                        if (affected != 0) {
                            var index = $scope.customers.indexOf($scope.customer);
                            $scope.customers.splice(index, 1);
                            $scope.customer = {};

                            $scope.$apply();
                        }
                    }, function (error) {
                        $ionicPopup.alert({ title: 'No se puedo eliminar el cliente', template: JSON.stringify(error) });
                        console.log(JSON.stringify(error));
                    });
                }
            });
        }

        /**
         *
         */
        $scope.show = function () {
            $scope.customerModal.show();
        }

        /**
         *
         */
        $scope.takePicture = function () {
            deviceFactory.takePicture(function (imageData) {
                $scope.customer.image_small = imageData;
                $scope.customer.image_medium = imageData;
            }, function (error) {
                console.log(error);
            });
        }

        /**
         *
         */
        $scope.addContact = function () {
            deviceFactory.saveContact($scope.customer, function (result) {
                console.log(result);
            }, function (error) {
                console.log(error);
            })
        }

        /**
         *
         */
        $scope.navigate = function () {
            deviceFactory.navigate([
                $scope.customer.partner_latitude,
                $scope.customer.partner_longitude
            ], function (message) {
                console.log(message);
            }, function (err) {
                console.log(err);
            })
        }
    });
